#ifndef FLUP_INTERPRETER_H
#define FLUP_INTERPRETER_H

#include "../submodules/utilitiec/src/dynamicarray/dynamicarray.h"
#include "../submodules/utilitiec/src/Scratchpad/Scratchpad.h"

#include "tokenizer.h"
#include <stdint.h>

typedef struct FlupFunctionAlternative_s {
	size_t condition_token_start;
	size_t condition_token_end;
	size_t body_token_start;
	size_t body_token_end;
	struct FlupFunctionAlternative_s* next;
} FlupFunctionAlternative;

typedef struct ParameterDefinition_s {
	StringView type;
	StringView name;
	struct ParameterDefinition_s* next;
} ParameterDefinition;

typedef struct FlupFunction_s {
	StringView name;
	ParameterDefinition* parameters;
	StringView return_type;
	FlupFunctionAlternative* alternative;
} FlupFunction;

enum ValueType {
	VALUETYPE_INT64,
	VALUETYPE_DOUBLE
};

union ValueContent {
	int64_t i64;
	double  f64;
};

typedef struct Value_s {
	enum ValueType type;
	union ValueContent get;
} Value;

typedef struct CallFrame_s {
	size_t instruction_pointer;
	/* ip = condition start : start
	 * ip = condition_end + 1 : done
	 *
	 */
	FlupFunctionAlternative* function;
	DynamicArray stack; // Value
	Scratchpad memory_pad;
} CallFrame;

typedef struct Interpreter_s {
	DynamicArray* tokens;
	DynamicArray call_frames; // stores CallFrame
} Interpreter;

int Interpreter_Create(Interpreter* self, DynamicArray* tokens);
int Interpreter_Interpret(Interpreter* self);
void Interpreter_Destroy(Interpreter* self);

#endif //header guard
